<?php
namespace nur\sery\wip\db\pdo;

use nur\sery\cl;

class Pdo {
  static function config_errmodeException_lowerCase(self $pdo) {
    $pdo->db->setAttribute(\PDO::ATTR_ERRMODE, \PDO::ERRMODE_EXCEPTION);
    $pdo->db->setAttribute(\PDO::ATTR_CASE, \PDO::CASE_LOWER);
  }

  const OPTIONS = [
    \PDO::ATTR_PERSISTENT => true,
  ];

  const CONFIG = [
    [self::class, "config_errmodeException_lowerCase"],
  ];

  const MIGRATE = null;

  const dbconn_SCHEMA = [
    "name" => "string",
    "user" => "?string",
    "pass" => "?string",
  ];

  const params_SCHEMA = [
    "options" => ["?array"],
    "config" => ["?array|callable"],
    "migrate" => ["?array|string|callable"],
    "auto_open" => ["bool", true],
  ];

  function __construct($dbconn, ?array $params=null) {
    if (!is_array($dbconn)) $dbconn = ["name" => $dbconn];
    $this->dbname = $dbconn["name"];
    $this->dbuser = $dbconn["user"] ?? null;
    $this->dbpass = $dbconn["pass"] ?? null;
    # options
    $this->options = cl::with($params["options"] ?? static::OPTIONS);
    # configuration
    $this->config = $params["config"] ?? static::CONFIG;
    # migrations
    $this->migration = $params["migrate"] ?? static::MIGRATE;
    #
    $defaultAutoOpen = self::params_SCHEMA["auto_open"][1];
    if ($params["auto_open"] ?? $defaultAutoOpen) {
      $this->open();
    }
  }

  protected ?string $dbconn;

  protected ?string $dbuser;

  protected ?string $dbpass;

  protected array $options;

  /** @var array|string|callable */
  protected $config;

  /** @var array|string|callable */
  protected $migration;

  protected ?\PDO $db;

  function open(): self {
    if ($this->db === null) {
      $this->db = new \PDO($this->dbname, $this->dbuser, $this->dbpass, $this->options);
      _config::with($this->config)->configure($this);
      _migration::with($this->migration)->migrate($this);
    }
    return $this;
  }

  function close(): void {
    $this->db = null;
  }
}
